/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2006 Smithsonian Astrophysical Observatory
 *
 * This file is part of projray
 *
 * src is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * src is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the 
 *       Free Software Foundation, Inc. 
 *       51 Franklin Street, Fifth Floor
 *       Boston, MA  02110-1301, USA
 *
 * -->8-->8-->8-->8-- */

#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#include <tracefct/tracefct.h>
#include <tracefct/exiterrvals.h>
#include <bpipe/bpipe.h>
#include <bpipe/datatypes.h>
#include <bpipe/bpipe_axaf.h>

#include <suplib/debug.h>

#include "main.h"
#include "common.h"
#include "mybpipe.h"


void
process(void)
{
  BPipe *bpipe;
  BPipeOutput *bpo;

  void *data;

  DVector3 *position, *direction;

  size_t dpkt_core_size;
  size_t nphot;

  int save_history = dbflag("save-history");

  tf_enter("process");

/* Setup bpipe input and output streams */

  if ( ( bpipe = bpipe_new() ) == NULL)
    tf_exit(ExitERR_misc, "couldn't open bpipe: %s",
	    bpipe_strerror(bpipe_errno));

  if ( 0 != bpipe_input(bpipe, c.input) )
    tf_exit(ExitERR_misc, "couldn't open input file `%s': %s", c.input,
	    bpipe_strerror(bpipe_errno));

  if ( NULL == ( bpo = bpipe_output( bpipe, c.output ) ) )
    tf_exit(ExitERR_misc, "couldn't open input file `%s': %s", c.output,
	    bpipe_strerror(bpipe_errno));

  if ( save_history )
  {
      dpktf_extend_1D( bpipe, BP_AXAF_phot_pos );
      dpktf_extend_1D( bpipe, BP_AXAF_phot_dir );
  }

  if (NULL == (data = bpipe_map_alloc(bpipe,1, &dpkt_core_size)))
    tf_exit( ExitERR_alloc, "unable to allocate memory for data packet: %s",
	    bpipe_strerror(bpipe_errno));

  position   = (DVector3 *) dpktf_data( bpipe, BP_AXAF_phot_pos, data);
  direction  = (DVector3 *) dpktf_data( bpipe, BP_AXAF_phot_dir, data);

  if ( bpipe_write_hdr(bpipe) )
    tf_exit(ExitERR_misc,"bpipe_write_hdr: %s",bpipe_strerror(bpipe_errno));

  nphot = 0;
  bpipe_errno = BPNOERROR;

  while( bpipe_read_dpkts( bpipe, data, (size_t) 1 )   == 1) 
  {

    double t;

    nphot++;

    /* if we're saving history, the actual input data are
       one up on the stack */
    if ( save_history )
    {
	*position  = *(position+1);
	*direction = *(direction+1);
    }

    t = (c.z - position->z) / direction->z;

    position->x += t * direction->x;
    position->y += t * direction->y;
    position->z = c.z;

/* Write the data packet out  */

    if(bpipe_write_dpkt( bpipe, data, bpo))
       tf_exit( ExitERR_misc, "error writing photon in projray %d", nphot); 
  }

  if ( bpipe_errno != BPNOERROR )
    tf_exit( ExitERR_misc, "error reading photon %d: %s", nphot + 1,
	     bpipe_strerror( bpipe_errno) );

  bpipe_delete( bpipe );

  tf_leave();
}

